use marble_engine::{
	board::Board,
	pos::{Pos, PosInt},
	tile::{Direction, Tile},
	Machine,
};
use raylib::{
	color::Color,
	drawing::{RaylibDraw, RaylibDrawHandle},
	math::Vector2,
};

use crate::{
	theme::TILE_TEXTURE_SIZE,
	ui::draw_usize_small,
	util::{draw_scaled_texture, Textures},
};

pub trait PosRaylib {
	fn to_vec(self) -> Vector2;
	fn from_vec(vec: Vector2) -> Self;
}

impl PosRaylib for Pos {
	fn from_vec(vec: Vector2) -> Self {
		Self {
			x: vec.x as PosInt,
			y: vec.y as PosInt,
		}
	}
	fn to_vec(self) -> Vector2 {
		Vector2 {
			x: self.x as f32,
			y: self.y as f32,
		}
	}
}

pub fn draw_board(
	board: &Board,
	d: &mut RaylibDrawHandle,
	textures: &Textures,
	offset: Vector2,
	scale: f32,
) {
	let tile_size = (TILE_TEXTURE_SIZE * scale) as i32;

	let start_x = (-offset.x as i32) / tile_size - 1;
	let tile_width = d.get_screen_width() / tile_size + 2;
	let start_y = (-offset.y as i32) / tile_size - 1;
	let tile_height = d.get_screen_height() / tile_size + 2;

	for x in start_x..(start_x + tile_width) {
		for y in start_y..(start_y + tile_height) {
			let px = x * tile_size + offset.x as i32;
			let py = y * tile_size + offset.y as i32;
			if let Some(tile) = board.get((x, y).into()) {
				let texname = tile.texture();
				if texname.is_empty() {
					continue;
				}
				let texture = textures.get(texname);
				draw_scaled_texture(d, texture, px, py, scale);
				#[cfg(debug_assertions)]
				// TODO: some in-game option to show power direction
				if let Tile::Powerable(_, state) = &tile {
					for dir in Direction::ALL {
						if state.get_dir(dir) {
							let texture = textures.get(dir.debug_arrow_texture_name());
							draw_scaled_texture(d, texture, px, py, scale);
						}
					}
				}
			} else {
				d.draw_rectangle(px, py, tile_size, tile_size, Color::new(0, 0, 0, 80));
			}
		}
	}
}

pub fn draw_marble_values(
	machine: &Machine,
	d: &mut RaylibDrawHandle,
	textures: &Textures,
	offset: Vector2,
	scale: f32,
) {
	let tile_size = (TILE_TEXTURE_SIZE * scale) as i32;
	for marble in machine.marbles() {
		let x = marble.x;
		let y = marble.y;
		if let Some(tile) = machine.board().get(*marble) {
			let px = x as i32 * tile_size + offset.x as i32;
			let py = y as i32 * tile_size + offset.y as i32;
			if let Tile::Marble { value, dir } = tile {
				let texture = textures.get(dir.arrow_texture_name());
				let pos = Vector2::new(px as f32, py as f32);
				let faded_white = Color::new(255, 255, 255, 100);
				d.draw_texture_ex(texture, pos, 0., scale, faded_white);
				draw_usize_small(d, textures, value as usize, px, py, scale);
			}
		}
	}
}
